==> create-fly-1.21.10-6.0.8-5-sources.jar: com/zurrtum/create/content/trains/graph/TrackNodeLocation.java <==
package com.zurrtum.create.content.trains.graph;

import com.mojang.serialization.*;
import com.zurrtum.create.catnip.data.Iterate;
import com.zurrtum.create.content.trains.track.BezierConnection;
import com.zurrtum.create.content.trains.track.TrackMaterial;
import io.netty.buffer.ByteBuf;
import java.util.*;
import java.util.stream.IntStream;
import net.minecraft.class_11368;
import net.minecraft.class_11372;
import net.minecraft.class_156;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2382;
import net.minecraft.class_243;
import net.minecraft.class_2540;
import net.minecraft.class_3532;
import net.minecraft.class_5321;
import net.minecraft.class_9135;
import net.minecraft.class_9139;

public class TrackNodeLocation extends class_2382 {
    private static final Codec<TrackNodeLocation> POS_CODEC = Codec.INT_STREAM.comapFlatMap(
        stream -> class_156.method_29190(stream, 3).map(coordinates -> new TrackNodeLocation(coordinates[0], coordinates[1], coordinates[2])),
        vec -> IntStream.of(vec.method_10263(), vec.method_10264(), vec.method_10260())
    );
    public static final class_9139<ByteBuf, TrackNodeLocation> POS_PACKET_CODEC = class_9139.method_56436(
        class_9135.field_48550,
        class_2382::method_10263,
        class_9135.field_48550,
        class_2382::method_10264,
        class_9135.field_48550,
        class_2382::method_10260,
        TrackNodeLocation::new
    );
    public class_5321<class_1937> dimension;
    public int yOffsetPixels;

    private TrackNodeLocation(int x, int y, int z) {
        super(x, y, z);
    }

    public TrackNodeLocation(class_243 vec) {
        this(vec.field_1352, vec.field_1351, vec.field_1350);
    }

    public TrackNodeLocation(double x, double y, double z) {
        super(class_3532.method_15357((double) Math.round(x * 2)), class_3532.method_15357(y) * 2, class_3532.method_15357((double) Math.round(z * 2)));
    }

    public TrackNodeLocation in(class_1937 level) {
        return in(level.method_27983());
    }

    public TrackNodeLocation in(class_5321<class_1937> dimension) {
        this.dimension = dimension;
        return this;
    }

    private static TrackNodeLocation fromPackedPos(class_2338 bufferPos) {
        return new TrackNodeLocation(bufferPos);
    }

    private TrackNodeLocation(class_2338 readBlockPos) {
        super(readBlockPos.method_10263(), readBlockPos.method_10264(), readBlockPos.method_10260());
    }

    public class_243 getLocation() {
        return new class_243(method_10263() / 2.0, method_10264() / 2.0 + yOffsetPixels / 16.0, method_10260() / 2.0);
    }

    public class_5321<class_1937> getDimension() {
        return dimension;
    }

    @Override
    public boolean equals(Object pOther) {
        return equalsIgnoreDim(pOther) && pOther instanceof TrackNodeLocation tnl && Objects.equals(tnl.dimension, dimension);
    }

    public boolean equalsIgnoreDim(Object pOther) {
        return super.equals(pOther) && pOther instanceof TrackNodeLocation tnl && tnl.yOffsetPixels == yOffsetPixels;
    }

    @Override
    public int hashCode() {
        return (method_10264() + ((method_10260() + yOffsetPixels * 31) * 31 + dimension.hashCode()) * 31) * 31 + method_10263();
    }

    public void write(class_11372 view, DimensionPalette dimensions) {
        view.method_71468("Pos", POS_CODEC, this);
        if (dimensions != null)
            view.method_71468("D", dimensions, dimension);
        if (yOffsetPixels != 0)
            view.method_71465("YO", yOffsetPixels);
    }

    public static <T> DataResult<T> encode(final TrackNodeLocation input, final DynamicOps<T> ops, final T empty, DimensionPalette dimensions) {
        RecordBuilder<T> builder = ops.mapBuilder();
        builder.add("Pos", input, POS_CODEC);
        if (dimensions != null)
            builder.add("D", input.dimension, dimensions);
        if (input.yOffsetPixels != 0)
            builder.add("YO", ops.createInt(input.yOffsetPixels));
        return builder.build(empty);
    }

    public static TrackNodeLocation read(class_11368 view, DimensionPalette dimensions) {
        TrackNodeLocation location = view.method_71426("Pos", POS_CODEC).orElseThrow();
        if (dimensions != null)
            location.dimension = view.method_71426("D", dimensions).orElse(null);
        location.yOffsetPixels = view.method_71424("YO", 0);
        return location;
    }

    public static <T> TrackNodeLocation decode(final DynamicOps<T> ops, final T input, DimensionPalette dimensions) {
        MapLike<T> map = ops.getMap(input).getOrThrow();
        TrackNodeLocation location = POS_CODEC.decode(ops, map.get("Pos")).getOrThrow().getFirst();
        if (dimensions != null)
            location.dimension = dimensions.parse(ops, map.get("D")).result().orElse(null);
        location.yOffsetPixels = Optional.ofNullable(map.get("YO")).map(value -> ops.getNumberValue(value).getOrThrow().intValue()).orElse(0);
        return location;
    }

    public void send(class_2540 buffer, DimensionPalette dimensions) {
        buffer.method_10804(method_10263());
        buffer.method_52998(method_10264());
        buffer.method_10804(method_10260());
        buffer.method_10804(yOffsetPixels);
        buffer.method_10804(dimensions.encode(dimension));
    }

    public static TrackNodeLocation receive(class_2540 buffer, DimensionPalette dimensions) {
        TrackNodeLocation location = new TrackNodeLocation(buffer.method_10816(), buffer.readShort(), buffer.method_10816());
        location.yOffsetPixels = buffer.method_10816();
        location.dimension = dimensions.decode(buffer.method_10816());
        return location;
    }

    public Collection<class_2338> allAdjacent() {
        Set<class_2338> set = new HashSet<>();
        class_243 vec3 = getLocation().method_1023(0, yOffsetPixels / 16.0, 0);
        double step = 1 / 8f;
        for (int x : Iterate.positiveAndNegative)
            for (int y : Iterate.positiveAndNegative)
                for (int z : Iterate.positiveAndNegative)
                    set.add(class_2338.method_49638(vec3.method_1031(x * step, y * step, z * step)));
        return set;
    }

    public static class DiscoveredLocation extends TrackNodeLocation {

        BezierConnection turn = null;
        boolean forceNode = false;
        class_243 direction;
        class_243 normal;
        TrackMaterial materialA;
        TrackMaterial materialB;

        public DiscoveredLocation(class_1937 level, double x, double y, double z) {
            super(x, y, z);
            in(level);
        }

        public DiscoveredLocation(class_5321<class_1937> dimension, class_243 vec) {
            super(vec);
            in(dimension);
        }

        public DiscoveredLocation(class_1937 level, class_243 vec) {
            this(level.method_27983(), vec);
        }

        public DiscoveredLocation materialA(TrackMaterial material) {
            this.materialA = material;
            return this;
        }

        public DiscoveredLocation materialB(TrackMaterial material) {
            this.materialB = material;
            return this;
        }

        public DiscoveredLocation materials(TrackMaterial materialA, TrackMaterial materialB) {
            this.materialA = materialA;
            this.materialB = materialB;
            return this;
        }

        public DiscoveredLocation viaTurn(BezierConnection turn) {
            this.turn = turn;
            if (turn != null)
                forceNode();
            return this;
        }

        public DiscoveredLocation forceNode() {
            forceNode = true;
            return this;
        }

        public DiscoveredLocation withNormal(class_243 normal) {
            this.normal = normal;
            return this;
        }

        public DiscoveredLocation withYOffset(int yOffsetPixels) {
            this.yOffsetPixels = yOffsetPixels;
            return this;
        }

        public DiscoveredLocation withDirection(class_243 direction) {
            this.direction = direction == null ? null : direction.method_1029();
            return this;
        }

        public boolean connectedViaTurn() {
            return turn != null;
        }

        public BezierConnection getTurn() {
            return turn;
        }

        public boolean shouldForceNode() {
            return forceNode;
        }

        public boolean differentMaterials() {
            return materialA != materialB;
        }

        public boolean notInLineWith(class_243 direction) {
            return this.direction != null && Math.max(
                direction.method_1026(this.direction),
                direction.method_1026(this.direction.method_1021(-1))
            ) < 7 / 8f;
        }

        public class_243 getDirection() {
            return direction;
        }

    }

}

==> create-fly-1.21.11-pre1-6.0.8-1-sources.jar: com/zurrtum/create/content/trains/graph/TrackNodeLocation.java <==
package com.zurrtum.create.content.trains.graph;

import com.mojang.serialization.*;
import com.zurrtum.create.catnip.data.Iterate;
import com.zurrtum.create.content.trains.track.BezierConnection;
import com.zurrtum.create.content.trains.track.TrackMaterial;
import io.netty.buffer.ByteBuf;

import java.util.*;
import java.util.stream.IntStream;
import net.minecraft.class_11368;
import net.minecraft.class_11372;
import net.minecraft.class_156;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2382;
import net.minecraft.class_243;
import net.minecraft.class_2540;
import net.minecraft.class_3532;
import net.minecraft.class_5321;
import net.minecraft.class_9135;
import net.minecraft.class_9139;

public class TrackNodeLocation extends class_2382 {
    private static final Codec<TrackNodeLocation> POS_CODEC = Codec.INT_STREAM.comapFlatMap(
        stream -> class_156.method_29190(stream, 3)
            .map(coordinates -> new TrackNodeLocation(coordinates[0], coordinates[1], coordinates[2])),
        vec -> IntStream.of(vec.method_10263(), vec.method_10264(), vec.method_10260())
    );
    public static final class_9139<ByteBuf, TrackNodeLocation> POS_PACKET_CODEC = class_9139.method_56436(
        class_9135.field_48550,
        class_2382::method_10263,
        class_9135.field_48550,
        class_2382::method_10264,
        class_9135.field_48550,
        class_2382::method_10260,
        TrackNodeLocation::new
    );
    public class_5321<class_1937> dimension;
    public int yOffsetPixels;

    private TrackNodeLocation(int x, int y, int z) {
        super(x, y, z);
    }

    public TrackNodeLocation(class_243 vec) {
        this(vec.field_1352, vec.field_1351, vec.field_1350);
    }

    public TrackNodeLocation(double x, double y, double z) {
        super(class_3532.method_15357((double) Math.round(x * 2)), class_3532.method_15357(y) * 2, class_3532.method_15357((double) Math.round(z * 2)));
    }

    public TrackNodeLocation in(class_1937 level) {
        return in(level.method_27983());
    }

    public TrackNodeLocation in(class_5321<class_1937> dimension) {
        this.dimension = dimension;
        return this;
    }

    private static TrackNodeLocation fromPackedPos(class_2338 bufferPos) {
        return new TrackNodeLocation(bufferPos);
    }

    private TrackNodeLocation(class_2338 readBlockPos) {
        super(readBlockPos.method_10263(), readBlockPos.method_10264(), readBlockPos.method_10260());
    }

    public class_243 getLocation() {
        return new class_243(method_10263() / 2.0, method_10264() / 2.0 + yOffsetPixels / 16.0, method_10260() / 2.0);
    }

    public class_5321<class_1937> getDimension() {
        return dimension;
    }

    @Override
    public boolean equals(Object pOther) {
        return equalsIgnoreDim(pOther) && pOther instanceof TrackNodeLocation tnl && Objects.equals(tnl.dimension, dimension);
    }

    public boolean equalsIgnoreDim(Object pOther) {
        return super.equals(pOther) && pOther instanceof TrackNodeLocation tnl && tnl.yOffsetPixels == yOffsetPixels;
    }

    @Override
    public int hashCode() {
        return (method_10264() + ((method_10260() + yOffsetPixels * 31) * 31 + dimension.hashCode()) * 31) * 31 + method_10263();
    }

    public void write(class_11372 view, DimensionPalette dimensions) {
        view.method_71468("Pos", POS_CODEC, this);
        if (dimensions != null)
            view.method_71468("D", dimensions, dimension);
        if (yOffsetPixels != 0)
            view.method_71465("YO", yOffsetPixels);
    }

    public static <T> DataResult<T> encode(final TrackNodeLocation input, final DynamicOps<T> ops, final T empty, DimensionPalette dimensions) {
        RecordBuilder<T> builder = ops.mapBuilder();
        builder.add("Pos", input, POS_CODEC);
        if (dimensions != null)
            builder.add("D", input.dimension, dimensions);
        if (input.yOffsetPixels != 0)
            builder.add("YO", ops.createInt(input.yOffsetPixels));
        return builder.build(empty);
    }

    public static TrackNodeLocation read(class_11368 view, DimensionPalette dimensions) {
        TrackNodeLocation location = view.method_71426("Pos", POS_CODEC).orElseThrow();
        if (dimensions != null)
            location.dimension = view.method_71426("D", dimensions).orElse(null);
        location.yOffsetPixels = view.method_71424("YO", 0);
        return location;
    }

    public static <T> TrackNodeLocation decode(final DynamicOps<T> ops, final T input, DimensionPalette dimensions) {
        MapLike<T> map = ops.getMap(input).getOrThrow();
        TrackNodeLocation location = POS_CODEC.decode(ops, map.get("Pos")).getOrThrow().getFirst();
        if (dimensions != null)
            location.dimension = dimensions.parse(ops, map.get("D")).result().orElse(null);
        location.yOffsetPixels = Optional.ofNullable(map.get("YO")).map(value -> ops.getNumberValue(value).getOrThrow().intValue()).orElse(0);
        return location;
    }

    public void send(class_2540 buffer, DimensionPalette dimensions) {
        buffer.method_10804(method_10263());
        buffer.method_52998(method_10264());
        buffer.method_10804(method_10260());
        buffer.method_10804(yOffsetPixels);
        buffer.method_10804(dimensions.encode(dimension));
    }

    public static TrackNodeLocation receive(class_2540 buffer, DimensionPalette dimensions) {
        TrackNodeLocation location = new TrackNodeLocation(buffer.method_10816(), buffer.readShort(), buffer.method_10816());
        location.yOffsetPixels = buffer.method_10816();
        location.dimension = dimensions.decode(buffer.method_10816());
        return location;
    }

    public Collection<class_2338> allAdjacent() {
        Set<class_2338> set = new HashSet<>();
        class_243 vec3 = getLocation().method_1023(0, yOffsetPixels / 16.0, 0);
        double step = 1 / 8f;
        for (int x : Iterate.positiveAndNegative)
            for (int y : Iterate.positiveAndNegative)
                for (int z : Iterate.positiveAndNegative)
                    set.add(class_2338.method_49638(vec3.method_1031(x * step, y * step, z * step)));
        return set;
    }

    public static class DiscoveredLocation extends TrackNodeLocation {

        BezierConnection turn = null;
        boolean forceNode = false;
        class_243 direction;
        class_243 normal;
        TrackMaterial materialA;
        TrackMaterial materialB;

        public DiscoveredLocation(class_1937 level, double x, double y, double z) {
            super(x, y, z);
            in(level);
        }

        public DiscoveredLocation(class_5321<class_1937> dimension, class_243 vec) {
            super(vec);
            in(dimension);
        }

        public DiscoveredLocation(class_1937 level, class_243 vec) {
            this(level.method_27983(), vec);
        }

        public DiscoveredLocation materialA(TrackMaterial material) {
            this.materialA = material;
            return this;
        }

        public DiscoveredLocation materialB(TrackMaterial material) {
            this.materialB = material;
            return this;
        }

        public DiscoveredLocation materials(TrackMaterial materialA, TrackMaterial materialB) {
            this.materialA = materialA;
            this.materialB = materialB;
            return this;
        }

        public DiscoveredLocation viaTurn(BezierConnection turn) {
            this.turn = turn;
            if (turn != null)
                forceNode();
            return this;
        }

        public DiscoveredLocation forceNode() {
            forceNode = true;
            return this;
        }

        public DiscoveredLocation withNormal(class_243 normal) {
            this.normal = normal;
            return this;
        }

        public DiscoveredLocation withYOffset(int yOffsetPixels) {
            this.yOffsetPixels = yOffsetPixels;
            return this;
        }

        public DiscoveredLocation withDirection(class_243 direction) {
            this.direction = direction == null ? null : direction.method_1029();
            return this;
        }

        public boolean connectedViaTurn() {
            return turn != null;
        }

        public BezierConnection getTurn() {
            return turn;
        }

        public boolean shouldForceNode() {
            return forceNode;
        }

        public boolean differentMaterials() {
            return materialA != materialB;
        }

        public boolean notInLineWith(class_243 direction) {
            return this.direction != null && Math.max(direction.method_1026(this.direction), direction.method_1026(this.direction.method_1021(-1))) < 7 / 8f;
        }

        public class_243 getDirection() {
            return direction;
        }

    }

}
